<a href="{{ url }}" data-rel="ajax" data-post="{{ data }}" class="cms-btn{% if active %} cms-btn-active{% endif %}{% if disabled %} cms-btn-disabled{% endif %}" data-on-success="{{ on_success }}">
    <span>
        <img
            aria-hidden="true" style="vertical-align: text-bottom"
            width="16" height="16" src="{{ icon }}" alt="{{ name }}"
            title="{{ name }}">
        <span class="cms-hidden" style="display: none;">{{ name }}</span>
    </span>
</a>
